#ifndef CORNER_TABLE_H
#define CORNER_TABLE_H

#include <stdlib.h>
#include <string.h>

#include "trianglemesh.h"
#include "vector.h"
#include "vector_int.h"

typedef struct {
	int v1;
	int v2;
} Edge;

typedef struct _DictionaryCTNode {
	Edge edge;
	int index;
	struct _DictionaryCTNode* next;
} DictionaryCTNode;

#define DICTIONARY_SIZE 1024

typedef struct {
	/* Dictionary is just array of arrays where index is determined by hash */
	DictionaryCTNode* nodes[DICTIONARY_SIZE];
} DictionaryCT;

#define CT_NODATA -1
#define CT_DIC_NODATA -1

/* Data structure which effectively processes neighbours
 * in trianglemesh */
typedef struct {
	int* opposite;
	const TriangleMesh* mesh;
	int* incident_corner;
} CornerTable;

int corner_table_next_corner(CornerTable* ct, int corner_index);
int corner_table_prev_corner(CornerTable* ct, int corner_index);
int corner_table_get_vertex(CornerTable* ct, int corner_index);

/* For vertex with index 'vertex' returns indexes of it's neightbours */
int* corner_table_neighbours(CornerTable* ct, int vertex, int* result_length);

CornerTable* corner_table_create(const TriangleMesh* mesh);

void corner_table_free(CornerTable* ct);

#endif
