#ifndef CURVATURE_ORACLE_H
#define CURVATURE_ORACLE_H

#include <pthread.h>
#include <math.h>

#include "neighbours_list.h"
#include "proximity_verifier.h"
#include "transform3d.h"
#include "k2tree.h"
#include "config.h"
#include "point_record.h"
#include "trianglemesh.h"
#include "neighbours_cache.h"
#include "stack_int.h"
#include "math_utils.h"
#include "vector.h"
#include "sampler.h"

typedef struct {
	int start;
	int n;

	PointRecord* points;
	int points_size;

	const NeighboursCache* cache;
	const TriangleMesh* mesh;
	int* samples;
	NeighboursList* list;

} CurvatureOraclePRArgs;

typedef struct {
	Transform3D* candidates;
	int size;
	int capacity;
} CandidatesTest;

typedef struct {
	int start;
	int n;

	int candidates_size;

	CandidatesTest* ct;

	K2Tree* tree;
	const ProximityVerifier* verifier;

	const NeighboursCache* cache;
	const TriangleMesh* mesh;
} CurvatureOracleFMArgs;

/* Finds candidate transformations by computing curvatures
 * of both sampled meshes and then mapping points with similar
 * curvatures to each other */
Transform3D* curvature_oracle_find_candidates(TriangleMesh* mesh1, TriangleMesh* mesh2, ProximityVerifier* verifier, int* candidates_size);

#endif
