#ifndef K2TREE_H
#define K2TREE_H

#include <stdlib.h>
#include <float.h>
#include <string.h>

#include "point_record.h"
#include "vector.h"

/* 2 dimensional k-d tree 
 * 
 * Because of the nature of the k-d tree way to compare/find points is dependant on
 * current depth of the search. For 2 dimensional there's constant switching between
 * comparing X values and Y values (in this case k1 and k2). Because of this fact
 * most of the functions are implemented via macros(in .c file). So the code can 
 * be dynamically generated. And there's no unnecessary copypasting. */

typedef struct _K2TreeNode {
	struct _K2TreeNode* left;
	struct _K2TreeNode* right;

	PointRecord point;
} K2TreeNode;

typedef struct {
	K2TreeNode* root;
} K2Tree;

K2TreeNode* k2tree_node_create();
PointRecord k2tree_find_nearest(K2Tree* tree, PointRecord p);
K2Tree* k2tree_create(PointRecord* points, int points_size);
void k2tree_free(K2Tree* tree);

#endif
