#ifndef POINT3D_H
#define POINT3D_H

#include <math.h>
#include <immintrin.h>

typedef union {
	struct {
		float x, y, z, _padding;
	};
	__m128 vector;
} Point3D;

inline Point3D create_point3d(float x, float y, float z) {
	Point3D point;

	point.x = x;
	point.y = y;
	point.z = z;

	return point;
}

inline void point_negative_ref(Point3D* p1) {
	p1->x = -p1->x;
	p1->y = -p1->y;
	p1->z = -p1->z;
}

inline Point3D POINTS_ADD(Point3D p1, Point3D p2) {
	Point3D res;

	res.vector = _mm_add_ps(p1.vector, p2.vector);

	return res;
}

inline void POINTS_SUB_REF(Point3D* p1, Point3D p2) {
	p1->vector = _mm_sub_ps(p1->vector, p2.vector);
}

inline Point3D POINTS_SUB(Point3D p1, Point3D p2) {
	Point3D res;

	res.vector = _mm_sub_ps(p1.vector, p2.vector);

	return res;
}

inline Point3D POINTS_CROSSPRODUCT(Point3D p1, Point3D p2) {
	Point3D res;

	res.x = p1.y * p2.z - p2.y * p1.z; 
	res.y = p1.z * p2.x - p2.z * p1.x;
	res.z = p1.x * p2.y - p2.x * p1.y;

	return res;
}

inline float POINTS_DOTPRODUCT(Point3D p1, Point3D p2) {
	return p1.x * p2.x + p1.y * p2.y + p1.z * p2.z;
}

inline float POINT_ABS(Point3D p) {
	return sqrt(p.x * p.x + p.y * p.y + p.z * p.z);
}

inline Point3D POINT_SCALE(Point3D p, float s) {
	Point3D res;

	res.x = p.x * s;
	res.y = p.y * s;
	res.z = p.z * s;

	return res;
}

inline void POINT_SCALE_REF(Point3D* p, float s) {
	p->x *= s;
	p->y *= s; 
	p->z *= s;
}

inline void POINTS_ADD_REF(Point3D* p1, Point3D p2) {
	p1->vector = _mm_add_ps(p1->vector, p2.vector);
}

inline float POINTS_DISTANCE(Point3D p1, Point3D p2) {
	return sqrt((p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y) * (p1.y - p2.y) + (p1.z - p2.z) * (p1.z - p2.z));
}

inline float points_distance_squared(Point3D p1, Point3D p2) {
	return (p1.x - p2.x) * (p1.x - p2.x) + (p1.y - p2.y) * (p1.y - p2.y) + (p1.z - p2.z) * (p1.z - p2.z);
}


void print_point3d(Point3D p);

//Point3D create_point3d(float x, float y, float z);

//inline void POINTS_ADD_REF(Point3D* p1, Point3D p2);
//define POINTS_ADD_REF(p1, p2) p1->x = p2.x; p1->y = p2.y; p1->z = p2.z;

//void POINTS_SUB_REF(Point3D* p1, Point3D p2);
//Point3D POINT_SCALE(Point3D p, float s);
//void POINT_SCALE_REF(Point3D* p, float s);
//
//Point3D POINTS_SUB(Point3D p1, Point3D p2);
//Point3D POINTS_CROSSPRODUCT(Point3D p1, Point3D p2);
//Point3D POINTS_ADD(Point3D p1, Point3D p2);
//
//void point_negative_ref(Point3D* p1);
//
//float POINTS_DOTPRODUCT(Point3D p1, Point3D p2);
//float POINT_ABS(Point3D p);
//float POINTS_DISTANCE(Point3D p1, Point3D p2);
//float points_distance_squared(Point3D p1, Point3D p2);

#endif
