#ifndef PROXIMITY_VERIFIER_H
#define PROXIMITY_VERIFIER_H

#include "trianglemesh.h"
#include "transform3d.h"
#include "config.h"
#include "point3d.h"
#include "config.h"

typedef struct {
	Point3D* points;
	int points_size;

	char* grid;
	int grid_x, grid_y, grid_z;
	Point3D min;

	char* kernel;
	int d_span, kernel_span;
	float step;
} T_Arguments_Static;

typedef struct {
	int start;
	int to_process;
	T_Arguments_Static* args;
} T_Arguments;

typedef struct {
	Point3D* mesh1;
	int mesh1_size;

	Point3D* mesh2;
	int mesh2_size;

	char* grid;
	int grid_x, grid_y, grid_z;

	float step;
	float inverse_step;
	int points_acceptance_threshold;

	/* Offset in space. This is used when mapping points to grid.
	 * min_point corresponds to grid[0]. */
	Point3D min_point;

} ProximityVerifier;

#define NUMBER_OF_CELLS 512

ProximityVerifier* create_proximity_verifier(TriangleMesh* mesh1, TriangleMesh* mesh2, float delta);
int proximity_verifier_verify(const ProximityVerifier* verifier, Transform3D transform);

void proximity_verifier_free(ProximityVerifier* verifier);

#endif
