#ifndef TRANSFORM3D_H
#define TRANSFORM3D_H

#include <string.h>
#include <stdio.h>

#include <pmmintrin.h>
#include <immintrin.h>

#include "point_record.h"
#include "point3d.h"
#include "matrix3x3.h"

typedef struct {
	float rotation[9];
	union {
		float translation[4];
		__m128 translation_vector;
	};
} Transform3D;

void transform3d_print(Transform3D t);
Transform3D transform3d_create_map(PointRecord source, PointRecord target, int swap);

inline Transform3D transform3d_create(float* rotation, float* translation) {
	Transform3D result;

	memcpy(result.rotation, rotation, sizeof(float) * 9);
	memcpy(result.translation, translation, sizeof(float) * 3);

	return result;
}

inline Point3D transform3d_transform_point_nonvector(Transform3D t, Point3D p) {
	float x = t.rotation[0] * p.x + t.rotation[1] * p.y + t.rotation[2] * p.z + t.translation[0];
	float y = t.rotation[3] * p.x + t.rotation[4] * p.y + t.rotation[5] * p.z + t.translation[1];
	float z = t.rotation[6] * p.x + t.rotation[7] * p.y + t.rotation[8] * p.z + t.translation[2];

	return create_point3d(x, y, z);
}

inline Point3D transform3d_transform_point(Transform3D t, Point3D p) {
	Point3D result;

	__m128 col0 = _mm_set_ps(0, t.rotation[6], t.rotation[3], t.rotation[0]);
	__m128 colp0 = _mm_set_ps1(p.x);
	__m128 res0 = _mm_mul_ps(col0, colp0);

	__m128 col1 = _mm_set_ps(0, t.rotation[7], t.rotation[4], t.rotation[1]);
	__m128 colp1 = _mm_set_ps1(p.y);
	__m128 res1 = _mm_mul_ps(col1, colp1);

	__m128 col2 = _mm_set_ps(0, t.rotation[8], t.rotation[5], t.rotation[2]);
	__m128 colp2 = _mm_set_ps1(p.z);
	__m128 res2 = _mm_mul_ps(col2, colp2);

	__m128 add01 = _mm_add_ps(res0, res1);
	__m128 add012 = _mm_add_ps(res2, add01);

	result.vector = _mm_add_ps(add012, t.translation_vector);

	return result;
}

inline void transform3d_transform_point_ref(Transform3D t, Point3D* p) {
	__m128 col0 = _mm_set_ps(0, t.rotation[6], t.rotation[3], t.rotation[0]);
	__m128 colp0 = _mm_set_ps1(p->x);
	__m128 res0 = _mm_mul_ps(col0, colp0);

	__m128 col1 = _mm_set_ps(0, t.rotation[7], t.rotation[4], t.rotation[1]);
	__m128 colp1 = _mm_set_ps1(p->y);
	__m128 res1 = _mm_mul_ps(col1, colp1);

	__m128 col2 = _mm_set_ps(0, t.rotation[8], t.rotation[5], t.rotation[2]);
	__m128 colp2 = _mm_set_ps1(p->z);
	__m128 res2 = _mm_mul_ps(col2, colp2);

	p->vector = _mm_add_ps(t.translation_vector, _mm_add_ps(res2, _mm_add_ps(res0, res1)));
}

inline void transform3d_transform_point_ref_translate_first(Transform3D t, Point3D* p) {
	/* FIRST translation */
	float tx = p->x + t.translation[0];
	float ty = p->y + t.translation[1];
	float tz = p->z + t.translation[2];

	/* then rotation */
	float x = t.rotation[0] * tx + t.rotation[1] * ty + t.rotation[2] * tz;
	float y = t.rotation[3] * tx + t.rotation[4] * ty + t.rotation[5] * tz;
	float z = t.rotation[6] * tx + t.rotation[7] * ty + t.rotation[8] * tz;

	p->x = x;
	p->y = y;
	p->z = z;
}

inline void transform3d_rotate_point_ref(Transform3D t, Point3D* p) {
	__m128 col0 = _mm_set_ps(0, t.rotation[6], t.rotation[3], t.rotation[0]);
	__m128 colp0 = _mm_set_ps(0, p->x, p->x, p->x);
	__m128 res0 = _mm_mul_ps(col0, colp0);

	__m128 col1 = _mm_set_ps(0, t.rotation[7], t.rotation[4], t.rotation[1]);
	__m128 colp1 = _mm_set_ps(0, p->y, p->y, p->y);
	__m128 res1 = _mm_mul_ps(col1, colp1);

	__m128 col2 = _mm_set_ps(0, t.rotation[8], t.rotation[5], t.rotation[2]);
	__m128 colp2 = _mm_set_ps(0, p->z, p->z, p->z);
	__m128 res2 = _mm_mul_ps(col2, colp2);

	p->vector = _mm_add_ps(res2, _mm_add_ps(res0, res1));
}

#endif
