#ifndef TRIANGLEMESH_H
#define TRIANGLEMESH_H

#include "point3d.h"
#include "triangle.h"

typedef struct {

	/* Array of vertexes */
	Point3D* points;
	/* Normals for vertexes(Average normal of triangles having i-th vertex) */
	Point3D* normals;
	int points_size;

	Triangle* triangles;
	int triangles_size;

	/* Array of vertex weights */
	float* weights;
	float weighted_radius, radius, max_area, avgEL, area_sum;

	Point3D centroid;

} TriangleMesh;

/* Allocates needed memory and fills attributes
 *
 * points: Array of vertexes
 * points_size: Size of array points 
 * triangles: Array of faces/triangles 
 * triangles_size: Size of array triangles */
TriangleMesh* create_trianglemesh(Point3D* points, int points_size, Triangle* triangles, int triangles_size);

/* Initialize triangle mesh that is - compute normals, radius, max_area and avgEL 
 *
 * mesh: Mesh to be initialized */
int init_trianglemesh(TriangleMesh* mesh);

void trianglemesh_init_radiuses(TriangleMesh* mesh);

void trianglemesh_free(TriangleMesh* mesh);

#endif
