#ifndef VECTOR_H
#define VECTOR_H

#ifndef VECTOR_INITIAL_SIZE
#define VECTOR_INITIAL_SIZE 32
#endif

/* Macros to represent resizable buffers - vectors / arraylists
 *
 * This solution might be considered distasteful by some C programmers
 * However i find this elegant and very convenient.
 * All it does is basically checking if the resize is needed every time
 * element is being added. If so it resizes array for float it's size.
 *
 * Usage:
 *
 * 		VECTOR_CREATE(int) integer_resizable_array;
 * 		VECTOR_INIT(integer_resizable_array, int);
 *
 * 		VECTOR_ADD(integer_resizable_array, 10);
 *
 * 		// Accessing data or size is done directly:
 *
 * 		int size = integer_resizable_array.size;
 * 		int first_element = integer_resizable_array.data[0];
 *
 * 		VECTOR_FREE(integer_resizable_array);
 */

/* Creates struct representing array of "TYPE" */
#define VECTOR_CREATE(TYPE)		\
struct { 						\
	TYPE* data;					\
	int size;					\
	int _size;					\
}

/* Fills */
#define VECTOR_INIT(VECTOR, TYPE)								\
	VECTOR.data = malloc(sizeof(TYPE) * VECTOR_INITIAL_SIZE); 	\
	VECTOR.size = 0;											\
	VECTOR._size = VECTOR_INITIAL_SIZE;

#define VECTOR_ADD(VECTOR, ELEMENT) 												\
	if(VECTOR.size >= VECTOR._size) {												\
		VECTOR._size *= 2;															\
		VECTOR.data = realloc(VECTOR.data, sizeof(typeof(ELEMENT)) * VECTOR._size);	\
	}																				\
	VECTOR.data[VECTOR.size] = ELEMENT;												\
	VECTOR.size += 1;

/* Only works for primitive arrays */
#define VECTOR_FREE(VECTOR)   \
	free(VECTOR.data);        \

#endif
