
#include <stddef.h>
#include "density_clusterer.h"

/* TODO: This is awkward. Should be passed somehow instead. (DensityClustererContext) */
static float spread;
static float radius;

void* density_clusterer_calculate_density_worker(void* arguments) {
	DCArgs args = *((DCArgs*) arguments);

	int start = args.start;
	int n = args.n;
	DistanceTree* tree = args.tree;
	Transform3D* candidates = args.candidates;
	int candidates_size = args.candidates_size;
	float* densities = args.densities;
	float threshold = args.threshold;

	if(start + n >= args.candidates_size) {
		n -= (start + n) - args.candidates_size;
	}

	for(int i = start; i < n + start; ++i) {
		int neighbours_size = 0;
		int* neighbours = distance_tree_find_close(tree, candidates[i], threshold, &neighbours_size);

		float density = 0;
		for(int j = 0; j < neighbours_size; ++j) {
			density += exp((-(spread * spread) * metric_compute_squared(candidates[i], candidates[neighbours[j]])) / (radius * radius));
		}

		densities[i] = density;
		free(neighbours);
	}
}

float* density_clusterer_start_workers(Transform3D* candidates, int candidates_size) {
	float* densities = malloc(sizeof(float) * candidates_size);
	float threshold = -radius * log(0.01) / spread;

	DistanceTree* tree = distance_tree_create(candidates, candidates_size);

	int number_of_processors = config_number_of_processors;
	int to_process = ceil((float) candidates_size / number_of_processors);

	pthread_t tid;
	pthread_t* tids = malloc(sizeof(pthread_t) * number_of_processors);
	DCArgs** t_arg_pointers = malloc(sizeof(DCArgs*) * number_of_processors);


	for(int i = 0; i < number_of_processors; ++i) {
		DCArgs* ta = malloc(sizeof(DCArgs));
		ta->start = i * to_process;
		ta->n = to_process;
		ta->tree = tree;
		ta->candidates = candidates;
		ta->candidates_size = candidates_size;
		ta->threshold = threshold;
		ta->densities = densities;

		pthread_create(&tid, NULL, density_clusterer_calculate_density_worker, (void*) ta);

		tids[i] = tid;
		t_arg_pointers[i] = ta;
	}

	for(int i = 0; i < number_of_processors; ++i) {
		pthread_join(tids[i], NULL);
	}

	for(int i = 0; i < number_of_processors; ++i) {
		free(t_arg_pointers[i]);
	}

	free(tids);
	free(t_arg_pointers);
	distance_tree_free(tree);

	return densities;
}

Transform3D density_clusterer_get_best_candidate(Transform3D* candidates, int candidates_size) {
	float* densities = density_clusterer_start_workers(candidates, candidates_size);

	float max_density = densities[0];
	int max_index = 0;

	for(int i = 1; i < candidates_size; ++i) {
		if(densities[i] > max_density) {
			max_density = densities[i];
			max_index = i;
		}
	}
	
	free(densities);
	return candidates[max_index];
}

void density_clusterer_init(float _spread, float _radius) {
	spread = _spread;
	radius = _radius;
}
