#include <stdlib.h>
#include "hashtable.h"

#define MAGIC1 100000007
#define MAGIC2 161803409
#define MAGIC3 423606823

int hashtable_key(HashTable* table, Tuple t) {
	return (MAGIC1 * t.c1 + MAGIC2 * t.c2 + MAGIC3 * t.c3) % table->size;
}

HashTable* create_hashtable(int size, float delta) {
	HashTable* table = malloc(sizeof(HashTable));

	if(table == NULL) {
		return NULL;
	}

	char* data = calloc(size, sizeof(char));
	Tuple* voxels = calloc(size, sizeof(Tuple));

	if(data == NULL || voxels == NULL) {
		free(table);
		return NULL;
	}

	table->data = data;
	table->size = size;
	table->voxels = voxels;
	table->scale = 1 / delta;

	return table;

}

int hashtable_add(HashTable* table, Point3D point) {
	Tuple t = {(int) (point.x * table->scale), (int) (point.y * table->scale), (int) (point.z * table->scale)};
	int key = hashtable_key(table, t);

	if(key < 0) {
		key += table->size;
	}

	while(1) {
		if(table->data[key] == HASH_NO_DATA) {
			table->voxels[key] = t;
			table->data[key] = HASH_DATA;

			return HASH_ADDED_NEW;
		}
		
		// If voxels are the same 
		if(table->voxels[key].c1 == t.c1 && table->voxels[key].c2 == t.c2 && table->voxels[key].c3 == t.c3) {
			return HASH_ADDED_EXISTING;
		}

		key += 1;
		key %= table->size;
	}
}

void free_hashtable(HashTable** table) {
	if(table == NULL || *table == NULL) {
		return;
	}

	if((*table)->data != NULL) {
		free((*table)->data);
	}

	if((*table)->voxels != NULL) {
		free((*table)->voxels);
	}

	free(*table);
	*table = NULL;
}
