
#include "k2tree.h"
#include <stdio.h>

K2TreeNode* k2tree_node_create() {
	K2TreeNode* node = malloc(sizeof(K2TreeNode));

	node->left = NULL;
	node->right = NULL;

	/* TODO: NULL record point if needed */

	return node;
}
//	qsort(ARR, ARR_SIZE, sizeof(PointRecord), CMP_FUNC);

/* Because of the reasons listed in header file macro
 * used in functions bodies: */
#define K2TREE_ADD_CHILDREN_BODY(NODE, ARR, ARR_SIZE, VAR, CMP_FUNC, NEXT_LEVEL_FUNC)     \
	if(ARR_SIZE == 1) {                                                                   \
		NODE->point = ARR[0];                                                             \
		free(ARR);                                                                        \
		return;                                                                           \
	}                                                                                     \
                                                                                          \
	qsort(ARR, ARR_SIZE, sizeof(PointRecord), CMP_FUNC);                                  \
                                                                                          \
	/* Median of the array is the point in the middle */                                  \
	int median_index = ARR_SIZE / 2;                                                      \
	NODE->point = ARR[median_index];                                                      \
                                                                                          \
	int left_size = median_index;                                                         \
	int right_size = ARR_SIZE - median_index - 1;                                         \
                                                                                          \
	/* TODO: Free the arrays(More concretely free "ARR" array) */                         \
                                                                                          \
	if(left_size > 0) {                                                                   \
		PointRecord* left_arr = malloc(sizeof(PointRecord) * left_size);                  \
		memcpy(left_arr, ARR, sizeof(PointRecord) * left_size);                           \
		K2TreeNode* left_node = k2tree_node_create();                                     \
		NODE->left = left_node;                                                           \
		NEXT_LEVEL_FUNC(left_node, left_arr, left_size);                                  \
	}                                                                                     \
                                                                                          \
	if(right_size > 0) {                                                                  \
		PointRecord* right_arr = malloc(sizeof(PointRecord) * right_size);                \
		memcpy(right_arr, &(ARR[median_index + 1]), sizeof(PointRecord) * right_size);    \
		K2TreeNode* right_node = k2tree_node_create();                                    \
		NODE->right = right_node;                                                         \
		NEXT_LEVEL_FUNC(right_node, right_arr, right_size);                               \
	}                                                                                     \
	free(ARR);

#define K2TREE_SEARCH_SUBTREE(NODE, VAR, POINT, DISTANCE, RESULT, NEXT_LEVEL_FUNC)         \
	float diff = POINT.VAR - NODE->POINT.VAR;                                             \
	float diff_absolute = fabs(diff);                                                     \
                                                                                           \
	if(diff_absolute < *DISTANCE) {                                                        \
		float new_distance = sqrt(point_record_distance_squared(NODE->point, POINT));     \
                                                                                           \
		if(new_distance < *DISTANCE) {                                                     \
			*RESULT = NODE->POINT;                                                         \
			*DISTANCE = new_distance;                                                      \
		}                                                                                  \
	}                                                                                      \
                                                                                           \
	if(diff < 0) {                                                                         \
		if(NODE->left != NULL) {                                                           \
			NEXT_LEVEL_FUNC(NODE->left, POINT, DISTANCE, RESULT);                          \
		}                                                                                  \
                                                                                           \
		if(NODE->right != NULL) {                                                          \
			if(diff_absolute < *DISTANCE) {                                                \
				NEXT_LEVEL_FUNC(NODE->right, POINT, DISTANCE, RESULT);                     \
			}                                                                              \
		}                                                                                  \
	} else {                                                                               \
		if(NODE->right != NULL) {                                                          \
			NEXT_LEVEL_FUNC(NODE->right, POINT, DISTANCE, RESULT);                         \
		}                                                                                  \
                                                                                           \
		if(NODE->left != NULL) {                                                           \
			if(diff_absolute < *DISTANCE) {                                                \
				NEXT_LEVEL_FUNC(NODE->left, POINT, DISTANCE, RESULT);                      \
			}                                                                              \
		}                                                                                  \
	}                                                                                      \


int k2tree_compare_k1(const void* a, const void* b) {
	PointRecord* pra = (PointRecord*) a;
	PointRecord* prb = (PointRecord*) b;

	if(pra->k1 == prb->k1) {
		return 0;
	}
	return pra->k1 > prb->k1 ? 1 : -1;
}

int k2tree_compare_k2(const void* a, const void* b) {
	PointRecord* pra = (PointRecord*) a;
	PointRecord* prb = (PointRecord*) b;

	if(pra->k2 == prb->k2) {
		return 0;
	}

	return pra->k2 > prb->k2 ? 1 : -1;
}

/* TODO: Maybe put these into header? */
void k2tree_add_children_k2(K2TreeNode* node, PointRecord* arr, int arr_size);
void k2tree_search_subtree_k2(K2TreeNode* node, PointRecord point, float* distance, PointRecord* result);

void k2tree_search_subtree_k1(K2TreeNode* node, PointRecord point, float* distance, PointRecord* result) {
	K2TREE_SEARCH_SUBTREE(node, k1, point, distance, result, k2tree_search_subtree_k2);
}

void k2tree_search_subtree_k2(K2TreeNode* node, PointRecord point, float* distance, PointRecord* result) {
	K2TREE_SEARCH_SUBTREE(node, k2, point, distance, result, k2tree_search_subtree_k1);
}

void k2tree_add_children_k1(K2TreeNode* node, PointRecord* arr, int arr_size) {
	K2TREE_ADD_CHILDREN_BODY(node, arr, arr_size, k1, k2tree_compare_k1, k2tree_add_children_k2);
}

void k2tree_add_children_k2(K2TreeNode* node, PointRecord* arr, int arr_size) {
	K2TREE_ADD_CHILDREN_BODY(node, arr, arr_size, k2, k2tree_compare_k2, k2tree_add_children_k1);
}

PointRecord k2tree_find_nearest(K2Tree* tree, PointRecord p) {
	float distance = DBL_MAX;
	PointRecord result = {0};

	k2tree_search_subtree_k1(tree->root, p, &distance, &result);

	return result;
}

K2Tree* k2tree_create(PointRecord* points, int points_size) {
	K2Tree* tree = malloc(sizeof(K2Tree));

	tree->root = k2tree_node_create();

	/* Copy of the points array is neededed, since points will be sorted */
	PointRecord* new_points = malloc(sizeof(PointRecord) * points_size);
	memcpy(new_points, points, points_size * sizeof(PointRecord));

	K2TreeNode* root = k2tree_node_create();
	tree->root = root;

	k2tree_add_children_k1(root, new_points, points_size);

	return tree;
}

void k2tree_free_rec(K2TreeNode* node) {
	if(node->left != NULL) {
		k2tree_free_rec(node->left);
	}

	if(node->right != NULL) {
		k2tree_free_rec(node->right);
	}

	free(node);
}

void k2tree_free(K2Tree* tree) {
	k2tree_free_rec(tree->root);
	free(tree);
}
