#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include "obj_loader.h"
#include "trianglemesh.h"
#include "triangle.h"
#include "point3d.h"
#include "transform3d.h"
#include "math_utils.h"
#include "cluster_registration.h"
#include "config.h"

#include "vector.h"
#include "time_measurement.h"

float config_delta_ratio = 0.01;
float config_sample_size_mesh1 = 2000;
float config_sample_size_mesh2 = 400;
float config_curvatures_calculated = 15000;
float config_candidates_length = 10000;
float config_neighbourhood_width = 8;
float config_rotation_coeff = 9.52;
float config_acceptace_threshold = 0.03;
float config_curvature_ratio = 0.7;
int config_number_of_processors;

/* TODO: Refactor whole project
 * 	- consistent names of functions
 * 	- magic values in curvature_oracle
 * 	- delete useless files
 * */
int main(int argc, char** argv) {

	clock_t p_begin, p_end;

	config_number_of_processors = sysconf(_SC_NPROCESSORS_ONLN);
	srand(time(0));

	p_begin = clock();

	TriangleMesh* mesh1 = get_trianglemesh_from_file(argv[1]);
	TriangleMesh* mesh2 = get_trianglemesh_from_file(argv[2]);

	if(mesh1 == NULL || mesh2 == NULL) {
		printf("Couldn't parse meshes. Exiting...\n");
		return EXIT_FAILURE;
	}

	init_trianglemesh(mesh1);
	init_trianglemesh(mesh2);

	Transform3D transform = cluster_registration_execute(mesh1, mesh2);
	transform3d_print(transform);

	p_end = clock();

	trianglemesh_free(mesh1);
	trianglemesh_free(mesh2);

	return EXIT_SUCCESS;
}
