#include <string.h>
#include <stdlib.h>
#include "matrix3x3.h"
#include "math_utils.h"
#include <stdio.h>
#include "point3d.h"

#define ROT(A, I, J, K, L, N) 			\
	k = A[J + I * N];			\
	l = A[L + K * N];			\
	A[J + I * N] = k - s * (l + k * tau);	\
	A[L + K * N] = l + s * (k - l * tau);

#define N_MAT 3

/* Taken from book: Numerical Recipes Third Edition Page 574.
 * Simplified/Modified for purpose of the algorithm */
int evd(Matrix3x3* m_mat, Matrix3x3* v_mat) {
	int i, j, ip, iq;
	float tresh, theta, tau, t, sm, s, h, g, c;
	float k, l;
	float a[N_MAT * N_MAT];
	float v[N_MAT * N_MAT] = {0};
	float b[N_MAT], z[N_MAT], d[N_MAT];
	memcpy(a, m_mat->data, sizeof(a));

	/* Identity matrix */
	v[0] = 1;
	v[4] = 1;
	v[8] = 1;

	/* Vectors b & d are now diagonal of matrix M */
	for(ip = 0; ip < N_MAT; ++ip) {
		b[ip] = a[ip + N_MAT * ip];
		d[ip] = b[ip];
		z[ip] = 0;
	} 

	/* ip is i-th row iq is i-th column */
	for(i = 1; i <= MAX_ITERATIONS; ++i) {
		sm = fabs(a[1]) + fabs(a[2]) + fabs(a[5]);

		if(sm == 0.0) {
			memcpy(v_mat->data, v, sizeof(v));
			return EXIT_SUCCESS;
		}

		if(i < 4) {
			tresh = 0.2 * sm / (N_MAT * N_MAT);
		} else {
			tresh = 0;
		}

		for(ip = 0; ip < N_MAT-1; ++ip) {
			for(iq = ip + 1; iq < N_MAT; ++iq) {
				g = 100 * fabs(iq + ip * N_MAT);
				if(i > 4 && g <= EPSILON * fabs(d[ip]) && g <= EPSILON * fabs(d[iq])) {
					a[iq + ip * N_MAT] = 0;
				}
				else if (fabs(a[iq + ip * N_MAT]) > tresh) {
					h = d[iq] - d[ip];
					if(g <= EPSILON * fabs(h)) {
						t = a[iq + ip * N_MAT] / h;
					} else {
						theta = 0.5 * h / (a[iq + ip * N_MAT]);
						t = 1.0 / (fabs(theta) + sqrt(1 + theta * theta));
						if(theta < 0) {
							t = -t;
						}
					}

					c = 1.0 / sqrt(1 + t * t);
					s = t * c;
					tau = s / (1.0 + c);
					h = t * a[iq + ip * N_MAT];

					z[ip] -= h;
					z[iq] += h;
					d[ip] -= h;
					d[iq] += h;

					a[iq + ip * N_MAT] = 0;

					for(j = 0; j < ip; ++j) {
						ROT(a, j, ip, j, iq, N_MAT);
					}
					for(j = ip + 1; j < iq; ++j) {
						ROT(a, ip, j, j, iq, N_MAT);
					}
					for(j = iq + 1; j < N_MAT; ++j) {
						ROT(a, ip, j, iq, j, N_MAT);
					}
					for(j = 0; j < N_MAT; ++j) {
						ROT(v, j, ip, j, iq, N_MAT);
					}
				}
			}
		}
		for(ip = 0; ip < N_MAT; ++ip) {
			b[ip] += z[ip];
			d[ip] = b[ip];
			z[ip] = 0;
		}
	}
	/* Too many iterations */
	return EXIT_FAILURE;
}
