
#include <stdlib.h>

#include "neighbours_cache.h"

NeighboursCache* neighbours_cache_create(int size, CornerTable* ct) {
	NeighboursCache* cache = malloc(sizeof(NeighboursCache));

	cache->size = size;
	cache->data = calloc(size, sizeof(int*));
	cache->data_sizes = calloc(size, sizeof(int));
	cache->corner_table = ct;

	return cache;
}

int* neighbours_cache_get(const NeighboursCache* cache, int index, int* result_length) {
	/* Index hasn't been accessed yet, need to cache it from corner table */
	if(cache->data[index] == NULL) {
		int len;
		int* neighbours = corner_table_neighbours(cache->corner_table, index, &len);

		cache->data[index] = neighbours;
		cache->data_sizes[index] = len;

		*result_length = len;
		return neighbours;
	}

	*result_length = cache->data_sizes[index];
	return cache->data[index];
}

void neighbours_cache_free(NeighboursCache** cache) {
	if(cache == NULL || *cache == NULL) {
		return;
	}

	for(int i = 0; i < (*cache)->size; ++i) {
		if((*cache)->data[i] != NULL) {
			free((*cache)->data[i]);
		}
	}

	free((*cache)->data_sizes);
	free((*cache)->data);
	free(*cache);
}
