#include <stdlib.h>

#include "sampler.h"
#include "hashtable.h"

int sampler_dist_uniform_sampling(Point3D* set, int set_size, int** sampled_set, int* sampled_size, float delta) {
	int size = 0;

	int* sampled = malloc(sizeof(int) * set_size);
	HashTable* table = create_hashtable(set_size, delta);

	if(sampled == NULL || table == NULL) {
		free(sampled);
		free_hashtable(&table);
		return 0;
	}

	for(int i = 0; i < set_size; ++i) {
		if(hashtable_add(table, set[i]) == HASH_ADDED_NEW) {
			sampled[size] = i;
			size += 1;
		} 	
	}

	free_hashtable(&table);

	*sampled_set = sampled;
	*sampled_size = size;

	return 1;
}

#define RANDOM_FLOAT (float) rand() / (float) RAND_MAX
void sampler_uniform_sampling_grid(TriangleMesh* mesh, int desired_size, float delta, int** sampled_points, int* sampled_points_size) {
	int* sampled_set;
	int sampled_size;

	sampler_dist_uniform_sampling(mesh->points, mesh->points_size, &sampled_set, &sampled_size, delta);

	*sampled_points = malloc(sizeof(int) * sampled_size);

	/* Taking every n-th(with p probability)
	 * sample from uniformly sampled mesh */
	int index = 0;
	float p = fmin(1, ((float) desired_size) / sampled_size);

	/* TODO: Instead of copypasting the code just 
	 * call sampler_sample from here */
	for(int i = 0; i < sampled_size; ++i) {
		if(RANDOM_FLOAT <= p) {
			(*sampled_points)[index] = i;
			index += 1;
		}
	}

	free(sampled_set);

	*sampled_points_size = index;
}

void sampler_sample(TriangleMesh* mesh, int desired_size, int** sampled_points, int* sampled_points_size) {

	*sampled_points = malloc(sizeof(int) * mesh->points_size);

	int index = 0;
	float p = fmin(1, ((float) desired_size) / mesh->points_size);

	/* Adding point with p probability */
	for(int i = 0; i < mesh->points_size; ++i) {
		if(RANDOM_FLOAT <= p) {
			(*sampled_points)[index] = i;
			index += 1;
		}
	}

	*sampled_points_size = index;
}
